set wdws [winfo children .]
if {[lsearch $wdws ".sd_0"] > 0} {
   echo "sd_0 already exists; choose another one"
   abort
}
set fnt "-*-Courier-Bold-R-Normal--*-100-*"
set SD_BITLEN [expr 7]
set sgnls [find signals /*]
set possible {}
foreach i $sgnls {
   set result [examine $i]
   if {([string length $result] == $SD_BITLEN) &&
       ([regexp -nocase "\[UX01ZWLH\-\]*" $i] == 1)} {
       lappend possible [file tail $i]
   }
}
if {[llength $possible] == 0} {
   echo "***** ERROR:no signal with bitlength $SD_BITLEN present *****"
   abort
}

toplevel .sd_0
canvas   .sd_0.cv -width 130 -height 200 -bg wheat -bd 2 -relief raised
frame    .sd_0.fr -bg gold
eval tk_optionMenu .sd_0.fr.opt sd_0_signals $possible
.sd_0.fr.opt configure -font $fnt
for {set i [expr 0]} {$i <= [.sd_0.fr.opt.menu index end]} {incr i} {
   .sd_0.fr.opt.menu entryconfigure $i -command "set_sd_0_signal $i"\
                                       -font $fnt
}

pack .sd_0.cv      -side top -fill both -expand 1
pack .sd_0.fr      -side bottom
pack .sd_0.fr.opt  -side left

tkwait visibility .sd_0.cv

bind .sd_0.cv <Configure> {upd_dim_sd_0 %w %h}
set WDTH [winfo width .sd_0.cv]
set HGHT [winfo height .sd_0.cv]
set dx [expr $WDTH/10.]
set dy [expr $HGHT/10.]
set x1 [expr $dx]
set x2 [expr 2*$dx]
set x3 [expr 3*$dx]
set x4 [expr $WDTH - $x3]
set x5 [expr $WDTH - $x2]
set x6 [expr $WDTH - $x1]
set y1 [expr $dy]
set y2 [expr $HGHT/2]
set y3 [expr $HGHT - $dy]
set line_width [expr 1+$WDTH/20]

.sd_0.cv create line $x3 $y1 $x6 $y1 -fill wheat2 -width $line_width\
                                     -capstyle round -tag seg0
.sd_0.cv create line $x6 $y1 $x5 $y2 -fill wheat2 -width $line_width\
                                     -capstyle round -tag seg1
.sd_0.cv create line $x5 $y2 $x4 $y3 -fill wheat2 -width $line_width\
                                     -capstyle round -tag seg2
.sd_0.cv create line $x4 $y3 $x1 $y3 -fill wheat2 -width $line_width\
                                     -capstyle round -tag seg3
.sd_0.cv create line $x1 $y3 $x2 $y2 -fill wheat2 -width $line_width\
                                     -capstyle round -tag seg4
.sd_0.cv create line $x2 $y2 $x3 $y1 -fill wheat2 -width $line_width\
                                     -capstyle round -tag seg5
.sd_0.cv create line $x2 $y2 $x5 $y2 -fill wheat2 -width $line_width\
                                     -capstyle round -tag seg6

when -label sd_0 /$sd_0_signals {
   set value [examine /$sd_0_signals]
   for {set i [expr 0]} {$i < $SD_BITLEN} {incr i} {
      if {[string index $value [expr $SD_BITLEN-$i-1]]== "1"} {
         .sd_0.cv raise seg$i
         .sd_0.cv itemconfigure seg$i -fill red
      }\
      else {
         .sd_0.cv lower seg$i
         .sd_0.cv itemconfigure seg$i -fill wheat2
      }
   }
   update
}

proc upd_dim_sd_0 {w_cv h_cv} {
   global topwdw

   set dx [expr $w_cv/10.]
   set dy [expr $h_cv/10.]
   set x1 [expr $dx]
   set x2 [expr 2*$dx]
   set x3 [expr 3*$dx]
   set x4 [expr $w_cv - $x3]
   set x5 [expr $w_cv - $x2]
   set x6 [expr $w_cv - $x1]
   set y1 [expr $dy]
   set y2 [expr $h_cv/2]
   set y3 [expr $h_cv - $dy]
   set line_width [expr 1+$w_cv/20.]
   .sd_0.cv coords seg0 $x3 $y1 $x6 $y1
   .sd_0.cv itemconfigure seg0 -width $line_width
   .sd_0.cv coords seg1 $x6 $y1 $x5 $y2
   .sd_0.cv itemconfigure seg1 -width $line_width
   .sd_0.cv coords seg2 $x5 $y2 $x4 $y3
   .sd_0.cv itemconfigure seg2 -width $line_width
   .sd_0.cv coords seg3 $x4 $y3 $x1 $y3
   .sd_0.cv itemconfigure seg3 -width $line_width
   .sd_0.cv coords seg4 $x1 $y3 $x2 $y2
   .sd_0.cv itemconfigure seg4 -width $line_width
   .sd_0.cv coords seg5 $x2 $y2 $x3 $y1
   .sd_0.cv itemconfigure seg5 -width $line_width
   .sd_0.cv coords seg6 $x2 $y2 $x5 $y2
   .sd_0.cv itemconfigure seg6 -width $line_width
}

proc set_sd_0_signal {idx} {
   global sd_0_signals

   nowhen sd_0

   set sd_0_signals [.sd_0.fr.opt.menu entrycget $idx -label]
   set value [examine /$sd_0_signals]
   set SD_BITLEN [string length $value]
   for {set i [expr 0]} {$i < $SD_BITLEN} {incr i} {
      if {[string index $value [expr $SD_BITLEN-$i-1]]== "1"} {
         .sd_0.cv itemconfigure seg$i -fill red
      }\
      else {
         .sd_0.cv itemconfigure seg$i -fill wheat2
      }
   }

   when -label sd_0 /$sd_0_signals {
      set value [examine /$sd_0_signals]
      for {set i [expr 0]} {$i < $SD_BITLEN} {incr i} {
         if {[string index $value [expr $SD_BITLEN-$i-1]]== "1"} {
            .sd_0.cv itemconfigure seg$i -fill red
         }\
         else {
            .sd_0.cv itemconfigure seg$i -fill wheat2
         }
      }
      update
   }
}







