entity equal_bit is
   port (in1: in  bit;
         in2: in  bit;
         res: out bit);
end equal_bit;

architecture fast of equal_bit is
begin
   res <= not (in1 xor in2) after 2 ns;
end fast;

architecture slow of equal_bit is
begin
   lbl1: process(in1, in2)
   begin
      if (in1 = in2) then
         res <= '1' after 10 ns;
      else
         res <= '0' after 10 ns;
      end if;
   end process;
end slow;


entity and_gate is
   port (ina:    in  bit;
         inb:    in  bit;
         result: out bit);
end and_gate;

architecture fast of and_gate is
begin
   result <= (ina and inb) after 5 ns;
end fast;

architecture slow of and_gate is
begin
   result <= (ina and inb) after 20 ns;
end slow;


entity comparator is
   port (a:  in  bit_vector(1 downto 0);
         b:  in  bit_vector(1 downto 0);
         eq: out bit);
end comparator;

architecture structural of comparator is
   component equal_bit
      port (in1: in  bit;
            in2: in  bit;
            res: out bit);
   end component;
   component and_gate
      port (ina:    in  bit;
            inb:    in  bit;
            result: out bit);
   end component;
   signal bit0, bit1: bit;
begin
   lbl1: equal_bit port map (a(0), b(0), bit0);
   lbl2: equal_bit port map (a(1), b(1), bit1);
   lbl3: and_gate  port map (bit0, bit1, eq);
end structural;

architecture behavioural of comparator is
begin
   lbl1: process(a, b)
   begin
      if (a = b) then
         eq <= '1' after 1 ns;
      else
         eq <= '0' after 1 ns;
      end if;
   end process;
end behavioural;


entity comparator_tb is
end comparator_tb;

architecture test of comparator_tb is
   component comparator
      port (a:  in  bit_vector(1 downto 0);
            b:  in  bit_vector(1 downto 0);
            eq: out bit);
   end component;
   signal aa, bb: bit_vector(1 downto 0);
   signal eq:     bit;
begin
   lbl1: comparator port map(aa, bb, eq);

   aa <= "00" after   0 ns,
         "01" after  50 ns,
         "10" after 100 ns,
         "11" after 150 ns,
         "00" after 200 ns,
         "01" after 250 ns,
         "10" after 300 ns,
         "11" after 350 ns,
         "00" after 400 ns,
         "01" after 450 ns,
         "10" after 500 ns,
         "11" after 550 ns,
         "00" after 600 ns,
         "01" after 650 ns,
         "10" after 700 ns,
         "11" after 750 ns,
         "00" after 800 ns;

   bb <= "00" after   0 ns,
         "01" after 200 ns,
         "10" after 400 ns,
         "11" after 600 ns,
         "00" after 800 ns;
end test;

----------------------------------------------------------------------
-- voltooi de volgende configuratie zodanig, dat de 'behavioural'
-- architecture van comparator wordt gebruikt bij simulatie.
----------------------------------------------------------------------
configuration comparator_tb_beh_cfg of comparator_tb is

end comparator_tb_beh_cfg;


-----------------------------------------------------------------------
-- voltooi de volgende configuratie zodanig, dat de 'structural'
-- architecture van comparator wordt gebruikt bij simulatie, en binnen
-- deze architecture de 'fast' versies van equal_bit en and_gate.
-----------------------------------------------------------------------
configuration comparator_tb_fast_cfg of comparator_tb is

end comparator_tb_fast_cfg;

------------------------------------------------------------------------
-- voltooi de volgende configuratie zodanig, dat de 'structural'
-- architecture van comparator wordt gebruikt bij simulatie, en binnen
-- deze architecture de 'slow' versies van equal_bit en and_gate.
-------------------------------------------------------------------------
configuration comparator_tb_slow_cfg of comparator_tb is

end comparator_tb_slow_cfg;










